@@ -36,7 +36,6 @@ EXCLUDE = (
     "cmd/smyrna/draw.c",
     "cmd/smyrna/draw.h",
     "cmd/smyrna/glexpose.c",
-    "cmd/smyrna/glexpose.h",
     "cmd/smyrna/glmotion.c",
     "cmd/smyrna/glmotion.h",
     "cmd/smyrna/gltemplate.c",
@@ -66,7 +65,6 @@ EXCLUDE = (
     "cmd/smyrna/smyrna_utils.c",
     "cmd/smyrna/smyrna_utils.h",
     "cmd/smyrna/smyrnadefs.h",
-    "cmd/smyrna/support.h",
     "cmd/smyrna/topfisheyeview.c",
     "cmd/smyrna/topfisheyeview.h",
     "cmd/smyrna/topviewfuncs.c",
@@ -79,7 +77,6 @@ EXCLUDE = (
     "cmd/smyrna/viewportcamera.h",
     "cmd/tools/acyclic.c",
     "cmd/tools/bcomps.c",
-    "cmd/tools/ccomps.c",
     "cmd/tools/colortbl.h",
     "cmd/tools/colxlate.c",
     "cmd/tools/convert.h",
@@ -123,7 +120,6 @@ EXCLUDE = (
     "lib/cdt/dtflatten.c",
     "lib/cdt/dthash.c",
     "lib/cdt/dthdr.h",
-    "lib/cdt/dtlist.c",
     "lib/cdt/dtmethod.c",
     "lib/cdt/dtopen.c",
     "lib/cdt/dtrenew.c",
@@ -144,7 +140,6 @@ EXCLUDE = (
     "lib/cgraph/ingraphs.c",
     "lib/cgraph/ingraphs.h",
     "lib/cgraph/io.c",
-    "lib/cgraph/mem.c",
     "lib/cgraph/node.c",
     "lib/cgraph/obj.c",
     "lib/cgraph/rec.c",
@@ -306,7 +301,6 @@ EXCLUDE = (
     "lib/gvc/gvrender.c",
     "lib/gvc/gvtextlayout.c",
     "lib/gvc/gvtool_tred.c",
-    "lib/gvc/gvusershape.c",
     "lib/gvpr/actions.h",
     "lib/gvpr/compile.h",
     "lib/gvpr/gdefs.h",
@@ -321,10 +315,8 @@ EXCLUDE = (
     "lib/label/node.c",
     "lib/label/node.h",
     "lib/label/rectangle.c",
-    "lib/label/rectangle.h",
     "lib/label/split.q.c",
     "lib/label/split.q.h",
-    "lib/label/xlabels.c",
     "lib/label/xlabels.h",
     "lib/mingle/agglomerative_bundling.cpp",
     "lib/mingle/edge_bundling.cpp",
@@ -393,7 +385,6 @@ EXCLUDE = (
     "lib/neatogen/quad_prog_vpsc.h",
     "lib/neatogen/randomkit.c",
     "lib/neatogen/randomkit.h",
-    "lib/neatogen/sgd.c",
     "lib/neatogen/sgd.h",
     "lib/neatogen/site.c",
     "lib/neatogen/site.h",
@@ -498,18 +489,27 @@ EXCLUDE = (
     "lib/vpsc/solve_VPSC.h",
     "lib/vpsc/variable.cpp",
     "lib/vpsc/variable.h",
-    "lib/xdot/xdot.c",
-    "lib/xdot/xdot.h",
     "macosx/GVApplicationDelegate.h",
+    "macosx/GVApplicationDelegate.m",
     "macosx/GVAttributeInspectorController.h",
+    "macosx/GVAttributeInspectorController.m",
     "macosx/GVAttributeSchema.h",
+    "macosx/GVAttributeSchema.m",
     "macosx/GVDocument.h",
+    "macosx/GVDocument.m",
     "macosx/GVExportViewController.h",
+    "macosx/GVExportViewController.m",
     "macosx/GVFileNotificationCenter.h",
+    "macosx/GVFileNotificationCenter.m",
     "macosx/GVGraphArguments.h",
+    "macosx/GVGraphArguments.m",
     "macosx/GVGraphDefaultAttributes.h",
+    "macosx/GVGraphDefaultAttributes.m",
     "macosx/GVWindowController.h",
+    "macosx/GVWindowController.m",
     "macosx/GVZGraph.h",
+    "macosx/GVZGraph.m",
+    "macosx/main.m",
     "plugin.demo/xgtk/src/callbacks.c",
     "plugin.demo/xgtk/src/callbacks.h",
     "plugin.demo/xgtk/src/gvdevice_xgtk.c",
@@ -555,7 +555,6 @@ EXCLUDE = (
     "plugin/pango/gvrender_pango.c",
     "plugin/pango/gvtextlayout_pango.c",
     "plugin/poppler/gvloadimage_poppler.c",
-    "plugin/quartz/GVTextLayout.h",
     "plugin/quartz/gvdevice_quartz.c",
     "plugin/quartz/gvloadimage_quartz.c",
     "plugin/quartz/gvplugin_quartz.c",
@@ -612,10 +611,12 @@ sources = subprocess.check_output(
         "**/*.c++",
         "**/*.cxx",
         "**/*.cc",
+        "**/*.m",
     ],
-    universal_newlines=True,
+    text=True,
 )
 
+ret = 0
 for source in sources.split("\x00")[:-1]:
     print(f"checking {source}...")
 
@@ -623,9 +624,9 @@ for source in sources.split("\x00")[:-1]:
 
     # ask `clang-format` to style the file
     reformatted = subprocess.check_output(
-        ["clang-format", "--style=file", "--", source],
+        ["clang-format-19", "--style=file", "--", source],
         cwd=root,
-        universal_newlines=True,
+        text=True,
     )
 
     if source in EXCLUDE:
@@ -647,4 +648,6 @@ for source in sources.split("\x00")[:-1]:
     )
     if diff:
         print(f"{source} incorrectly formatted:\n{''.join(diff)}", end="")
-        sys.exit(-1)
+        ret = -1
+
+sys.exit(ret)
