@@ -3,7 +3,7 @@
  * All rights reserved. This program and the accompanying materials
  * are made available under the terms of the Eclipse Public License v1.0
  * which accompanies this distribution, and is available at
- * http://www.eclipse.org/legal/epl-v10.html
+ * https://www.eclipse.org/legal/epl-v10.html
  *
  * Contributors: Details at https://graphviz.org
  *************************************************************************/
@@ -17,55 +17,55 @@
 
 
 #include "config.h"
+#include <limits.h>
+#include <stddef.h>
+#include <stdio.h>
 
-#ifdef HAVE_UNISTD_H
-#include <unistd.h>
-#else
-#include <ast/compat_unistd.h>
-#endif
-
-
-
+#include <cgraph/cgraph.h>
 #include <gvpr/gvpr.h>
+#include <util/exit.h>
 
 #ifdef DEBUG
-#include <sfio/sfio.h>
 static ssize_t outfn (void* sp, const char *buf, size_t nbyte, void* dp)
 {
-    write (1, "<stdout> ", 8);
-    return write (1, buf, nbyte);
+  if (nbyte > INT_MAX) {
+    return -1;
+  }
+  return printf("<stdout>%.*s", (int)nbyte, buf);
 }
 
 static ssize_t errfn (void* sp, const char *buf, size_t nbyte, void* dp)
 {
-    write (2, "<stderr> ", 8);
-    return write (2, buf, nbyte);
+  if (nbyte > INT_MAX) {
+    return -1;
+  }
+  return fprintf(stderr, "<stderr>%.*s", (int)nbyte, buf);
 }
 
 static int iofread(void *chan, char *buf, int bufsize)
 {
-    return read(sffileno((Sfio_t *) chan), buf, bufsize);
+  return (int)fread(buf, 1, (size_t)bufsize, chan);
 }
 
 static int ioputstr(void *chan, const char *str)
 {
-    return sfputr((Sfio_t *) chan, str, -1);
+  return fputs(str, chan);
 }
 
 static int ioflush(void *chan)
 {
-    return sfsync((Sfio_t *) chan);
+  return fflush(chan);
 }
 
 static Agiodisc_t gprIoDisc = { iofread, ioputstr, ioflush };
 
-static Agdisc_t gprDisc = { &AgMemDisc, &AgIdDisc, &gprIoDisc };
+static Agdisc_t gprDisc = { &AgIdDisc, &gprIoDisc };
 
 int
 main (int argc, char* argv[])
 {
     Agraph_t* gs[2];
-    Agraph_t* g = agread (sfstdin, &gprDisc);
+    Agraph_t* g = agread(stdin, &gprDisc);
     int rv;
     gvpropts opts;
 
@@ -79,13 +79,11 @@ main (int argc, char* argv[])
     
     rv = gvpr (argc, argv, &opts);
 
-    sfprintf (sfstderr, "rv %d\n", rv);
-    /* for (i = 0; i < opts.n_outgraphs; i++) */
-	/* agwrite (opts.outgraphs[i], sfstdout); */
+    fprintf(stderr, "rv %d\n", rv);
 
     rv = gvpr (argc, argv, &opts);
 
-    return rv;
+    graphviz_exit(rv);
 }
 
 #else
@@ -99,9 +97,12 @@ main (int argc, char* argv[])
     opts.flags = GV_USE_EXIT;
     opts.bindings = 0;
     
-    
-    return gvpr (argc, argv, &opts);
+    graphviz_exit(gvpr(argc, argv, &opts));
 }
 
 #endif
 
+/**
+ * @dir cmd/gvpr
+ * @brief graph pattern scanning and processing language
+ */
