@@ -21,13 +21,14 @@ XDOT DRAWING FUNCTIONS, maybe need to move them somewhere else
 #include <stdbool.h>
 #include <stddef.h>
 #include <stdlib.h>
+#include <string.h>
 #include <util/unreachable.h>
 #include <util/xml.h>
 
 #include <xdot/xdot.h>
 #include "viewport.h"
 #include "topfisheyeview.h"
-#include "appmouse.h"
+#include "gui/appmouse.h"
 #include "hotkeymap.h"
 #include "polytess.h"
 #include <glcomp/glcompimage.h>
@@ -288,10 +289,9 @@ static void InsertImage(xdot_op *op, int param) {
 static int put(void *buffer, const char *s) {
   char **b = buffer;
 
-  for (; *s != '\0'; ++s) {
-    **b = *s;
-    ++(*b);
-  }
+  const size_t len = strlen(s);
+  memcpy(*b, s, len);
+  *b += len;
 
   return 0;
 }
