@@ -41,10 +41,12 @@ static void check_cycles(graph_t * g);
 #define SEQ(a,b,c)		((a) <= (b) && (b) <= (c))
 #define TREE_EDGE(e)	(ED_tree_index(e) >= 0)
 
+DEFINE_LIST(node_list, node_t *)
+DEFINE_LIST(edge_list, edge_t *)
+
 typedef struct {
     graph_t *G;
-    nlist_t Tree_node;
-    elist Tree_edge;
+    edge_list_t Tree_edge;
     size_t S_i;			/* search index for enter_edge */
     size_t N_edges, N_nodes;
     int Search_size;
@@ -53,7 +55,7 @@ typedef struct {
     int Low, Lim, Slack;
 } network_simplex_ctx_t;
 
-#define SEARCHSIZE 30
+enum { SEARCHSIZE = 30 };
 
 static int add_tree_edge(network_simplex_ctx_t *ctx, edge_t * e)
 {
@@ -62,13 +64,9 @@ static int add_tree_edge(network_simplex_ctx_t *ctx, edge_t * e)
 	agerrorf("add_tree_edge: missing tree edge\n");
 	return -1;
     }
-    assert(ctx->Tree_edge.size <= INT_MAX);
-    ED_tree_index(e) = (int)ctx->Tree_edge.size;
-    ctx->Tree_edge.list[ctx->Tree_edge.size++] = e;
-    if (!ND_mark(agtail(e)))
-	ctx->Tree_node.list[ctx->Tree_node.size++] = agtail(e);
-    if (!ND_mark(aghead(e)))
-	ctx->Tree_node.list[ctx->Tree_node.size++] = aghead(e);
+    assert(edge_list_size(&ctx->Tree_edge) <= INT_MAX);
+    ED_tree_index(e) = (int)edge_list_size(&ctx->Tree_edge);
+    edge_list_append(&ctx->Tree_edge, e);
     n = agtail(e);
     ND_mark(n) = true;
     ND_tree_out(n).list[ND_tree_out(n).size++] = e;
@@ -122,7 +120,8 @@ static void exchange_tree_edges(network_simplex_ctx_t *ctx, edge_t * e, edge_t *
     node_t *n;
 
     ED_tree_index(f) = ED_tree_index(e);
-    ctx->Tree_edge.list[ED_tree_index(e)] = f;
+    assert(ED_tree_index(e) >= 0);
+    edge_list_set(&ctx->Tree_edge, (size_t)ED_tree_index(e), f);
     ED_tree_index(e) = -1;
 
     n = agtail(e);
@@ -193,13 +192,13 @@ static edge_t *leave_edge(network_simplex_ctx_t *ctx)
     int cnt = 0;
 
     size_t j = ctx->S_i;
-    while (ctx->S_i < ctx->Tree_edge.size) {
-	if (ED_cutvalue(f = ctx->Tree_edge.list[ctx->S_i]) < 0) {
+    while (ctx->S_i < edge_list_size(&ctx->Tree_edge)) {
+	if (ED_cutvalue(f = edge_list_get(&ctx->Tree_edge, ctx->S_i)) < 0) {
 	    if (rv) {
 		if (ED_cutvalue(rv) > ED_cutvalue(f))
 		    rv = f;
 	    } else
-		rv = ctx->Tree_edge.list[ctx->S_i];
+		rv = edge_list_get(&ctx->Tree_edge, ctx->S_i);
 	    if (++cnt >= ctx->Search_size)
 		return rv;
 	}
@@ -208,12 +207,12 @@ static edge_t *leave_edge(network_simplex_ctx_t *ctx)
     if (j > 0) {
 	ctx->S_i = 0;
 	while (ctx->S_i < j) {
-	    if (ED_cutvalue(f = ctx->Tree_edge.list[ctx->S_i]) < 0) {
+	    if (ED_cutvalue(f = edge_list_get(&ctx->Tree_edge, ctx->S_i)) < 0) {
 		if (rv) {
 		    if (ED_cutvalue(rv) > ED_cutvalue(f))
 			rv = f;
 		} else
-		    rv = ctx->Tree_edge.list[ctx->S_i];
+		    rv = edge_list_get(&ctx->Tree_edge, ctx->S_i);
 		if (++cnt >= ctx->Search_size)
 		    return rv;
 	    }
@@ -637,7 +636,7 @@ end:
   for (size_t i = 0; i < subtree_count; i++) free(tree[i]);
   free(tree);
   if (error) return error;
-  assert(ctx->Tree_edge.size == ctx->N_nodes - 1);
+  assert(edge_list_size(&ctx->Tree_edge) == ctx->N_nodes - 1);
   init_cutvalues(ctx);
   return 0;
 }
@@ -745,12 +744,7 @@ static int scan_and_normalize(network_simplex_ctx_t *ctx) {
 }
 
 static void reset_lists(network_simplex_ctx_t *ctx) {
-
-  free(ctx->Tree_node.list);
-  ctx->Tree_node = (nlist_t){0};
-
-  free(ctx->Tree_edge.list);
-  ctx->Tree_edge = (elist){0};
+  edge_list_free(&ctx->Tree_edge);
 }
 
 static void
@@ -770,8 +764,8 @@ static void LR_balance(network_simplex_ctx_t *ctx)
     int delta;
     edge_t *e, *f;
 
-    for (size_t i = 0; i < ctx->Tree_edge.size; i++) {
-	e = ctx->Tree_edge.list[i];
+    for (size_t i = 0; i < edge_list_size(&ctx->Tree_edge); i++) {
+	e = edge_list_get(&ctx->Tree_edge, i);
 	if (ED_cutvalue(e) == 0) {
 	    f = enter_edge(ctx,e);
 	    if (f == NULL)
@@ -788,7 +782,7 @@ static void LR_balance(network_simplex_ctx_t *ctx)
     freeTreeList(ctx, ctx->G);
 }
 
-static int decreasingrankcmpf(const void *x, const void *y) {
+static int decreasingrankcmpf(const node_t **x, const node_t **y) {
 // Suppress Clang/GCC -Wcast-qual warning. Casting away const here is acceptable
 // as the later usage is const. We need the cast because the macros use
 // non-const pointers for genericity.
@@ -810,26 +804,8 @@ static int decreasingrankcmpf(const void *x, const void *y) {
   return 0;
 }
 
-static int increasingrankcmpf(const void *x, const void *y) {
-// Suppress Clang/GCC -Wcast-qual warning. Casting away const here is acceptable
-// as the later usage is const. We need the cast because the macros use
-// non-const pointers for genericity.
-#ifdef __GNUC__
-#pragma GCC diagnostic push
-#pragma GCC diagnostic ignored "-Wcast-qual"
-#endif
-  node_t **n0 = (node_t **)x;
-  node_t **n1 = (node_t **)y;
-#ifdef __GNUC__
-#pragma GCC diagnostic pop
-#endif
-  if (ND_rank(*n0) < ND_rank(*n1)) {
-    return -1;
-  }
-  if (ND_rank(*n0) > ND_rank(*n1)) {
-    return 1;
-  }
-  return 0;
+static int increasingrankcmpf(const node_t **x, const node_t **y) {
+  return -decreasingrankcmpf(x, y);
 }
 
 static void TB_balance(network_simplex_ctx_t *ctx)
@@ -859,20 +835,20 @@ static void TB_balance(network_simplex_ctx_t *ctx)
                 }
               }
     }
-    size_t ii;
-    for (ii = 0, n = GD_nlist(ctx->G); n; ii++, n = ND_next(n)) {
-      ctx->Tree_node.list[ii] = n;
+    node_list_t Tree_node = {0};
+    node_list_reserve(&Tree_node, ctx->N_nodes);
+    for (n = GD_nlist(ctx->G); n; n = ND_next(n)) {
+      node_list_append(&Tree_node, n);
     }
-    ctx->Tree_node.size = ii;
-    qsort(ctx->Tree_node.list, ctx->Tree_node.size, sizeof(ctx->Tree_node.list[0]),
-          adj > 1 ? decreasingrankcmpf: increasingrankcmpf);
-    for (size_t i = 0; i < ctx->Tree_node.size; i++) {
-        n = ctx->Tree_node.list[i];
+    node_list_sort(&Tree_node,
+                   adj > 1 ? decreasingrankcmpf: increasingrankcmpf);
+    for (size_t i = 0; i < node_list_size(&Tree_node); i++) {
+        n = node_list_get(&Tree_node, i);
         if (ND_node_type(n) == NORMAL)
           nrank[ND_rank(n)]++;
     }
-    for (ii = 0; ii < ctx->Tree_node.size; ii++) {
-      n = ctx->Tree_node.list[ii];
+    for (size_t ii = 0; ii < node_list_size(&Tree_node); ii++) {
+      n = node_list_get(&Tree_node, ii);
       if (ND_node_type(n) != NORMAL)
         continue;
       inweight = outweight = 0;
@@ -907,6 +883,7 @@ static void TB_balance(network_simplex_ctx_t *ctx)
       free_list(ND_tree_out(n));
       ND_mark(n) = false;
     }
+    node_list_free(&Tree_node);
     free(nrank);
 }
 
@@ -923,8 +900,7 @@ static bool init_graph(network_simplex_ctx_t *ctx, graph_t *g) {
 	    ctx->N_edges++;
     }
 
-    ctx->Tree_node.list = gv_calloc(ctx->N_nodes, sizeof(node_t *));
-    ctx->Tree_edge.list = gv_calloc(ctx->N_nodes, sizeof(edge_t *));
+    edge_list_reserve(&ctx->Tree_edge, ctx->N_nodes);
 
     bool feasible = true;
     for (n = GD_nlist(g); n; n = ND_next(n)) {
@@ -1330,7 +1306,7 @@ void tchk(network_simplex_ctx_t *ctx)
 		fprintf(stderr, "not a tight tree %p", e);
 	}
     }
-    if (n_cnt != ctx->Tree_node.size || e_cnt != ctx->Tree_edge.size)
+    if (e_cnt != edge_list_size(&ctx->Tree_edge))
 	fprintf(stderr, "something missing\n");
 }
 
