@@ -10,14 +10,15 @@
  * Contributors: Details at https://graphviz.org
  *************************************************************************/
 
-#include <cgraph/alloc.h>
-#include <cgraph/agxbuf.h>
-#include <cgraph/prisize_t.h>
-#include <cgraph/unreachable.h>
 #include <common/render.h>
+#include <float.h>
 #include <label/xlabels.h>
 #include <stdbool.h>
 #include <stddef.h>
+#include <util/agxbuf.h>
+#include <util/alloc.h>
+#include <util/prisize_t.h>
+#include <util/unreachable.h>
 
 static int Rankdir;
 static bool Flip;
@@ -299,9 +300,7 @@ static void
 addXLabel (textlabel_t* lp, object_t* objp, xlabel_t* xlp, int initObj, pointf pos)
 {
     if (initObj) {
-	objp->sz.x = 0;
-	objp->sz.y = 0;
-	objp->pos = pos;
+	*objp = (object_t){.pos = pos};
     }
 
     if (Flip) {
@@ -393,15 +392,13 @@ static size_t countClusterLabels(Agraph_t *g) {
     return i;
 }
 
-/* addXLabels:
- * Position xlabels and any unpositioned edge labels using
- * a map placement algorithm to avoid overlap.
- *
- * TODO: interaction with spline=ortho
- */
   /* True if edges geometries were computed and this edge has a geometry */
 #define HAVE_EDGE(ep) ((et != EDGETYPE_NONE) && (ED_spl(ep) != NULL))
 
+/// position xlabels and any unpositioned edge labels using a map placement
+/// algorithm to avoid overlap
+///
+/// TODO: interaction with spline=ortho
 static void addXLabels(Agraph_t * gp)
 {
     Agnode_t *np;
@@ -472,8 +469,8 @@ static void addXLabels(Agraph_t * gp)
     size_t n_objs = (size_t)agnnodes(gp) + n_set_lbls + n_clbls + n_elbls;
     object_t* objp = objs = gv_calloc(n_objs, sizeof(object_t));
     xlabel_t* xlp = lbls = gv_calloc(n_lbls, sizeof(xlabel_t));
-    bb.LL = (pointf){INT_MAX, INT_MAX};
-    bb.UR = (pointf){-INT_MAX, -INT_MAX};
+    bb.LL = (pointf){DBL_MAX, DBL_MAX};
+    bb.UR = (pointf){-DBL_MAX, -DBL_MAX};
 
     for (np = agfstnode(gp); np; np = agnxtnode(gp, np)) {
 
