@@ -26,58 +26,127 @@
 #include	<dotgen/dot.h>
 #include	<limits.h>
 #include	<stdbool.h>
-#include	<stddef.h>
+#include	<stdlib.h>
 #include	<stdint.h>
 #include	<util/alloc.h>
+#include	<util/list.h>
 #include	<util/gv_math.h>
 
 static void dot1_rank(graph_t *g);
 static void dot2_rank(graph_t *g);
 
-static void 
-renewlist(elist * L)
-{
-    for (size_t i = L->size; i != SIZE_MAX; i--)
+DEFINE_LIST(edge_set, edge_t *)
+
+/// @param track An optional collection in which to record non-null entries we
+///   removed
+static void renewlist(elist *L, edge_set_t *track) {
+    for (size_t i = L->size; i != SIZE_MAX; i--) {
+	if (track != NULL && L->list[i] != NULL) {
+            edge_set_append(track, L->list[i]);
+	}
 	L->list[i] = NULL;
+    }
     L->size = 0;
 }
 
+/// compare two edge pointers
+///
+/// This function assumes the two edge pointers may have differing provenance,
+/// and thus cannot be directly compared. It also assumes the caller is
+/// primarily comparing in order to put equal elements next to each other in a
+/// sorting operation. Relying on, e.g. an earlier allocated edge pointer to
+/// compare as less than a later allocated edge pointer is not a good idea.
+///
+/// @param a One edge
+/// @param b Another edge
+/// @return A comparator value suitable for sorting algorithms
+static int edge_ptr_cmp(const edge_t **a, const edge_t **b) {
+  const uintptr_t addr_a = (uintptr_t)*a;
+  const uintptr_t addr_b = (uintptr_t)*b;
+  if (addr_a < addr_b) {
+    return -1;
+  }
+  if (addr_a > addr_b) {
+    return 1;
+  }
+  return 0;
+}
+
 static void 
 cleanup1(graph_t * g)
 {
-    node_t *n;
     edge_t *e, *f;
 
+    edge_set_t to_free = {0};
+
     for (size_t c = 0; c < GD_comp(g).size; c++) {
 	    GD_nlist(g) = GD_comp(g).list[c];
-	    for (n = GD_nlist(g); n; n = ND_next(n)) {
-	        renewlist(&ND_in(n));
-	        renewlist(&ND_out(n));
+	    for (node_t *n = GD_nlist(g), *next, *prev = NULL; n; n = next) {
+	        next = ND_next(n);
+	        // out edges are owning, so only track their removal
+	        renewlist(&ND_in(n), NULL);
+	        renewlist(&ND_out(n), &to_free);
 	        ND_mark(n) = false;
+	        // If this is a slack node, it exists _only_ in the component lists
+	        // that we are about to drop. Remove and deallocate slack nodes now to
+	        // avoid leaking these.
+	        if (ND_node_type(n) == SLACKNODE) {
+                if (prev == NULL) {
+                    GD_comp(g).list[c] = next;
+                    GD_nlist(g) = next;
+                } else {
+                    ND_next(prev) = next;
+                }
+                if (next != NULL) {
+                    ND_prev(next) = prev;
+                }
+                free_list(ND_in(n));
+                free_list(ND_out(n));
+                free(n->base.data);
+                free(n);
+	        } else {
+                prev = n;
+	        }
 	    }
     }
-    for (n = agfstnode(g); n; n = agnxtnode(g, n)) {
+    for (node_t *n = agfstnode(g); n; n = agnxtnode(g, n)) {
         for (e = agfstout(g, n); e; e = agnxtout(g, e)) {
             f = ED_to_virt(e);
             /* Null out any other references to f to make sure we don't
              * handle it a second time. For example, parallel multiedges
              * share a virtual edge.
              */
-            if (f && (e != ED_to_orig(f))) {
+            if (f && e != ED_to_orig(f)) {
                 ED_to_virt(e) = NULL;
             }
         }
     }
-    for (n = agfstnode(g); n; n = agnxtnode(g, n)) {
+    for (node_t *n = agfstnode(g); n; n = agnxtnode(g, n)) {
         for (e = agfstout(g, n); e; e = agnxtout(g, e)) {
             f = ED_to_virt(e);
             if (f && ED_to_orig(f) == e) {
-		        free(f->base.data);
-		        free(f);
+                edge_set_append(&to_free, f);
                 ED_to_virt(e) = NULL;
             }
 	    }
     }
+
+    // free all the edges we removed
+    // XXX: Accruing all the pointers, including duplicates, and then sorting to
+    // avoid duplicate frees is suboptimal. If this turns out to be a
+    // performance problem, replace `edge_set_t` with a proper set.
+    edge_set_sort(&to_free, edge_ptr_cmp);
+    edge_t *previous = NULL;
+    for (size_t i = 0; i < edge_set_size(&to_free); ++i) {
+        edge_t *const current = edge_set_get(&to_free, i);
+        if (current != previous) {
+            free(current->base.data);
+            free(current);
+        }
+        previous = current;
+    }
+    edge_set_free(&to_free);
+
     free(GD_comp(g).list);
     GD_comp(g).list = NULL;
     GD_comp(g).size = 0;
