@@ -3,7 +3,7 @@
  * All rights reserved. This program and the accompanying materials
  * are made available under the terms of the Eclipse Public License v1.0
  * which accompanies this distribution, and is available at
- * http://www.eclipse.org/legal/epl-v10.html
+ * https://www.eclipse.org/legal/epl-v10.html
  *
  * Contributors: Details at https://graphviz.org
  *************************************************************************/
@@ -12,6 +12,7 @@
 
 #include	<stdbool.h>
 #include	<stddef.h>
+#include	<stdio.h>
 #include	<string.h>
 #include	<unistd.h>
 
@@ -19,9 +20,8 @@
 #include	<ltdl.h>
 #endif
 
-#include	<cgraph/agxbuf.h>
-#include        <common/memory.h>
 #include        <common/types.h>
+#include        <gvc/gvc.h>
 #include        <gvc/gvplugin.h>
 #include        <gvc/gvcjob.h>
 #include        <gvc/gvcint.h>
@@ -29,10 +29,12 @@
 #include        <gvc/gvio.h>
 
 #include	<common/const.h>
-#include <cgraph/alloc.h>
-#include <cgraph/list.h>
-#include <cgraph/strcasecmp.h>
-#include <cgraph/strview.h>
+#include <util/agxbuf.h>
+#include <util/alloc.h>
+#include <util/list.h>
+#include <util/startswith.h>
+#include <util/strcasecmp.h>
+#include <util/strview.h>
 
 /*
  * Define an apis array of name strings using an enumerated api_t as index.
@@ -109,7 +111,7 @@ bool gvplugin_install(GVC_t *gvc, api_t api, const char *typestr, int quality,
         pnext = &(*pnext)->next;
     }
 
-    plugin = GNEW(gvplugin_available_t);
+    plugin = gv_alloc(sizeof(gvplugin_available_t));
     plugin->next = *pnext;
     *pnext = plugin;
     plugin->typestr = t;
@@ -148,15 +150,12 @@ static void gvplugin_activate(GVC_t * gvc, api_t api, const char *typestr,
     }
 }
 
-gvplugin_library_t *gvplugin_library_load(GVC_t * gvc, char *path)
-{
+gvplugin_library_t *gvplugin_library_load(GVC_t *gvc, const char *pathname) {
 #ifdef ENABLE_LTDL
     lt_dlhandle hndl;
     lt_ptr ptr;
-    char *s, *sym;
+    char *s;
     size_t len;
-    static char *p;
-    static size_t lenp;
     char *libdir;
     char *suffix = "_LTX_library";
 
@@ -164,35 +163,32 @@ gvplugin_library_t *gvplugin_library_load(GVC_t * gvc, char *path)
         return NULL;
 
     libdir = gvconfig_libdir(gvc);
-    len = strlen(libdir) + 1 + strlen(path) + 1;
-    if (len > lenp) {
-        lenp = len + 20;
-        p = grealloc(p, lenp);
-    }
+    agxbuf fullpath = {0};
 #ifdef _WIN32
-    if (path[1] == ':') {
+    if (pathname[1] == ':') {
 #else
-    if (path[0] == '/') {
+    if (pathname[0] == '/') {
 #endif
-        strcpy(p, path);
+        agxbput(&fullpath, pathname);
     } else {
-        strcpy(p, libdir);
-        strcat(p, DIRSEP);
-        strcat(p, path);
+        agxbprint(&fullpath, "%s%s%s", libdir, DIRSEP, pathname);
     }
 
     if (lt_dlinit()) {
-        agerr(AGERR, "failed to init libltdl\n");
+        agerrorf("failed to init libltdl\n");
+        agxbfree(&fullpath);
         return NULL;
     }
+    char *p = agxbuse(&fullpath);
     hndl = lt_dlopen(p);
     if (!hndl) {
         if (access(p, R_OK) == 0) {
-            agerr(AGWARN, "Could not load \"%s\" - %s\n", p, "It was found, so perhaps one of its dependents was not.  Try ldd.");
+            agwarningf("Could not load \"%s\" - %s\n", p, "It was found, so perhaps one of its dependents was not.  Try ldd.");
         }
         else {
-            agerr(AGWARN, "Could not load \"%s\" - %s\n", p, lt_dlerror());
+            agwarningf("Could not load \"%s\" - %s\n", p, lt_dlerror());
         }
+        agxbfree(&fullpath);
         return NULL;
     }
     if (gvc->common.verbose >= 2)
@@ -205,10 +201,11 @@ gvplugin_library_t *gvplugin_library_load(GVC_t * gvc, char *path)
 #else
     if (len < strlen("/libgvplugin_x")) {
 #endif
-        agerr(AGERR, "invalid plugin path \"%s\"\n", p);
+        agerrorf("invalid plugin path \"%s\"\n", p);
+        agxbfree(&fullpath);
         return NULL;
     }
-    sym = gmalloc(len + strlen(suffix) + 1);
+    char *sym = gv_alloc(len + strlen(suffix) + 1);
 #if defined(_WIN32) && !defined(__MINGW32__) && !defined(__CYGWIN__)
     strcpy(sym, s + 1);         /* strip leading "/"  */
 #else
@@ -223,14 +220,19 @@ gvplugin_library_t *gvplugin_library_load(GVC_t * gvc, char *path)
 
     ptr = lt_dlsym(hndl, sym);
     if (!ptr) {
-        agerr(AGERR, "failed to resolve %s in %s\n", sym, p);
+        agerrorf("failed to resolve %s in %s\n", sym, p);
         free(sym);
+        agxbfree(&fullpath);
         return NULL;
     }
     free(sym);
+    agxbfree(&fullpath);
     return (gvplugin_library_t *)ptr;
 #else
-    agerr(AGERR, "dynamic loading not available\n");
+    (void)gvc;
+    (void)pathname;
+
+    agerrorf("dynamic loading not available\n");
     return NULL;
 #endif
 }
@@ -248,8 +250,8 @@ gvplugin_library_t *gvplugin_library_load(GVC_t * gvc, char *path)
 		png:gd:gd
       
 */
-gvplugin_available_t *gvplugin_load(GVC_t * gvc, api_t api, const char *str)
-{
+gvplugin_available_t *gvplugin_load(GVC_t *gvc, api_t api, const char *str,
+                                    FILE *debug) {
     gvplugin_available_t *pnext, *rv;
     gvplugin_library_t *library;
     gvplugin_api_t *apis;
@@ -276,6 +278,8 @@ gvplugin_available_t *gvplugin_load(GVC_t * gvc, api_t api, const char *str)
         }
     }
 
+    agxbuf diag = {0}; // diagnostic messages
+
     /* iterate the linked list of plugins for this api */
     for (pnext = gvc->apis[api]; pnext; pnext = pnext->next) {
         const strview_t typ = strview(pnext->typestr, ':');
@@ -285,10 +289,17 @@ gvplugin_available_t *gvplugin_load(GVC_t * gvc, api_t api, const char *str)
             dep = strview(typ.data + typ.size + strlen(":"), '\0');
         }
 
-        if (!strview_eq(typ, reqtyp))
+        if (!strview_eq(typ, reqtyp)) {
+            agxbprint(&diag, "# type \"%.*s\" did not match \"%.*s\"\n",
+                      (int)typ.size, typ.data, (int)reqtyp.size, reqtyp.data);
             continue;           /* types empty or mismatched */
+        }
         if (dep.data && reqdep.data) {
             if (!strview_eq(dep, reqdep)) {
+                agxbprint(&diag,
+                          "# dependencies \"%.*s\" did not match \"%.*s\"\n",
+                          (int)dep.size, dep.data, (int)reqdep.size,
+                          reqdep.data);
                 continue;           /* dependencies not empty, but mismatched */
             }
         }
@@ -296,8 +307,12 @@ gvplugin_available_t *gvplugin_load(GVC_t * gvc, api_t api, const char *str)
             // found with no packagename constraints, or with required matching packagename
 
             if (dep.data && apidep != api) // load dependency if needed, continue if can't find
-                if (!gvplugin_load(gvc, apidep, dep.data))
+                if (!gvplugin_load(gvc, apidep, dep.data, debug)) {
+                    agxbprint(&diag,
+                              "# plugin loading of dependency \"%.*s\" failed\n",
+                              (int)dep.size, dep.data);
                     continue;
+                }
             break;
         }
     }
@@ -321,12 +336,19 @@ gvplugin_available_t *gvplugin_load(GVC_t * gvc, api_t api, const char *str)
     }
 
     /* one last check for successful load */
-    if (rv && rv->typeptr == NULL)
+    if (rv && rv->typeptr == NULL) {
+        agxbprint(&diag, "# unsuccessful plugin load\n");
         rv = NULL;
+    }
 
     if (rv && gvc->common.verbose >= 1)
         fprintf(stderr, "Using %s: %s:%s\n", api_names[api], rv->typestr, rv->package->name);
 
+    if (debug != NULL) {
+        fputs(agxbuse(&diag), debug);
+    }
+    agxbfree(&diag);
+
     gvc->api[api] = rv;
     return rv;
 }
@@ -355,8 +377,19 @@ char *gvplugin_list(GVC_t * gvc, api_t api, const char *str)
                                           then just list the alternative paths for the plugin */
         for (pnext = plugin; pnext; pnext = pnext->next) {
             const strview_t type = strview(pnext->typestr, ':');
-            /* list only the matching type, or all types if s is an empty string */
-            if (!str[0] || strview_case_eq(strv, type)) {
+            // skip duplicates
+            bool already_seen = false;
+            for (const gvplugin_available_t *p = plugin; p != pnext;
+                 p = p->next) {
+                already_seen |= strcasecmp(pnext->typestr, p->typestr) == 0 &&
+                  strcasecmp(pnext->package->name, p->package->name) == 0;
+            }
+            if (already_seen) {
+                continue;
+            }
+            // list only the matching type, or all types if str is an empty
+            // string or starts with ":"
+            if (strv.size == 0 || strview_case_eq(strv, type)) {
                 /* list each member of the matching type as "type:path" */
                 agxbprint(&xb, " %s:%s", pnext->typestr, pnext->package->name);
                 new = false;
@@ -393,18 +426,12 @@ DEFINE_LIST(strs, char*)
  * freeing each item, then the list.
  * Returns NULL on error, or if there are no plugins.
  * In the former case, sz is unchanged; in the latter, sz = 0.
- *
- * At present, the str argument is unused, but may be used to modify
- * the search as in gvplugin_list above.
  */
-char **gvPluginList(GVC_t * gvc, const char *kind, int *sz, const char *str)
-{
+char **gvPluginList(GVC_t *gvc, const char *kind, int *sz) {
     size_t api;
     const gvplugin_available_t *pnext, *plugin;
     strs_t list = {0};
 
-    (void)str;
-
     if (!kind)
         return NULL;
     for (api = 0; api < ARRAY_SIZE(api_names); api++) {
@@ -412,7 +439,7 @@ char **gvPluginList(GVC_t * gvc, const char *kind, int *sz, const char *str)
             break;
     }
     if (api == ARRAY_SIZE(api_names)) {
-        agerr(AGERR, "unrecognized api name \"%s\"\n", kind);
+        agerrorf("unrecognized api name \"%s\"\n", kind);
         return NULL;
     }
 
@@ -465,19 +492,19 @@ Agraph_t *gvplugin_graph(GVC_t * gvc)
     Agsym_t *a;
     gvplugin_package_t *package;
     const gvplugin_available_t *pnext;
-    char bufa[100], *buf1, *buf2, bufb[100], *p, *q, *lq, *t;
+    char *p, *q, *lq, *t;
     int neededge_loadimage, neededge_device;
 
     g = agopen("G", Agdirected, NULL);
-    agattr(g, AGRAPH, "label", "");
-    agattr(g, AGRAPH, "rankdir", "");
-    agattr(g, AGRAPH, "rank", "");
-    agattr(g, AGRAPH, "ranksep", "");
-    agattr(g, AGNODE, "label", NODENAME_ESC);
-    agattr(g, AGNODE, "shape", "");
-    agattr(g, AGNODE, "style", "");
-    agattr(g, AGNODE, "width", "");
-    agattr(g, AGEDGE, "style", "");
+    agattr_text(g, AGRAPH, "label", "");
+    agattr_text(g, AGRAPH, "rankdir", "");
+    agattr_text(g, AGRAPH, "rank", "");
+    agattr_text(g, AGRAPH, "ranksep", "");
+    agattr_text(g, AGNODE, "label", NODENAME_ESC);
+    agattr_text(g, AGNODE, "shape", "");
+    agattr_text(g, AGNODE, "style", "");
+    agattr_text(g, AGNODE, "width", "");
+    agattr_text(g, AGEDGE, "style", "");
 
     a = agfindgraphattr(g, "rankdir");
     agxset(g, a, "LR");
@@ -488,24 +515,19 @@ Agraph_t *gvplugin_graph(GVC_t * gvc)
     a = agfindgraphattr(g, "label");
     agxset(g, a, "Plugins");
 
+    agxbuf buf = {0};
     for (package = gvc->packages; package; package = package->next) {
         loadimage_n = renderer_n = device_n = textlayout_n = layout_n = NULL;
         neededge_loadimage = neededge_device = 0;
-        strcpy(bufa, "cluster_");
-        strcat(bufa, package->name);
-        sg = agsubg(g, bufa, 1);
+        agxbprint(&buf, "cluster_%s", package->name);
+        sg = agsubg(g, agxbuse(&buf), 1);
         a = agfindgraphattr(sg, "label");
         agxset(sg, a, package->name);
-        strcpy(bufa, package->name);
-        strcat(bufa, "_");
-        buf1 = bufa + strlen(bufa);
         for (size_t api = 0; api < ARRAY_SIZE(api_names); api++) {
-            strcpy(buf1, api_names[api]);
-            ssg = agsubg(sg, bufa, 1);
+            agxbprint(&buf, "%s_%s", package->name, api_names[api]);
+            ssg = agsubg(sg, agxbuse(&buf), 1);
             a = agfindgraphattr(ssg, "rank");
             agxset(ssg, a, "same");
-            strcat(buf1, "_");
-            buf2 = bufa + strlen(bufa);
             for (pnext = gvc->apis[api]; pnext; pnext = pnext->next) {
                 if (pnext->package == package) {
                     t = q = gv_strdup(pnext->typestr);
@@ -522,11 +544,11 @@ Agraph_t *gvplugin_graph(GVC_t * gvc)
 
                         /* hack for aliases */
 			lq = q;
-                        if (!strncmp(q, "jp", 2)) {
+                        if (startswith(q, "jp")) {
                             q = "jpg";                /* canonical - for node name */
 			    lq = "jpeg\\njpe\\njpg";  /* list - for label */
 			}
-                        else if (!strncmp(q, "tif", 3)) {
+                        else if (startswith(q, "tif")) {
                             q = "tif";
 			    lq = "tiff\\ntif";
 			}
@@ -539,8 +561,9 @@ Agraph_t *gvplugin_graph(GVC_t * gvc)
                             lq = "gv\\ndot";
 			}
 
-                        strcpy(buf2, q);
-                        n = agnode(ssg, bufa, 1);
+                        agxbprint(&buf, "%s_%s_%s", package->name,
+                                  api_names[api], q);
+                        n = agnode(ssg, agxbuse(&buf), 1);
                         a = agfindnodeattr(g, "label");
                         agxset(n, a, lq);
                         a = agfindnodeattr(g, "width");
@@ -555,10 +578,9 @@ Agraph_t *gvplugin_graph(GVC_t * gvc)
                             loadimage_n = n;
 			}
                         if (!(p && *p)) {
-                            strcpy(bufb, "render_cg");
-                            m = agfindnode(sg, bufb);
+                            m = agfindnode(sg, "render_cg");
                             if (!m) {
-                                m = agnode(sg, bufb, 1);
+                                m = agnode(sg, "render_cg", 1);
                                 a = agfindgraphattr(g, "label");
                                 agxset(m, a, "cg");
                             }
@@ -567,20 +589,16 @@ Agraph_t *gvplugin_graph(GVC_t * gvc)
                         break;
                     case API_render:
 			/* draw renderers as ellipses - record last renderer in plugin (if any) in renderer_n */
-                        strcpy(bufb, api_names[api]);
-                        strcat(bufb, "_");
-                        strcat(bufb, q);
-                        renderer_n = n = agnode(ssg, bufb, 1);
+                        agxbprint(&buf, "%s_%s", api_names[api], q);
+                        renderer_n = n = agnode(ssg, agxbuse(&buf), 1);
                         a = agfindnodeattr(g, "label");
                         agxset(n, a, q);
                         break;
                     case API_textlayout:
 			/* draw textlayout  as invtriangle - record last textlayout in plugin  (if any) in textlayout_n */
 			/* FIXME? only one textlayout is loaded. Why? */
-                        strcpy(bufb, api_names[api]);
-                        strcat(bufb, "_");
-                        strcat(bufb, q);
-                        textlayout_n = n = agnode(ssg, bufb, 1);
+                        agxbprint(&buf, "%s_%s", api_names[api], q);
+                        textlayout_n = n = agnode(ssg, agxbuse(&buf), 1);
                         a = agfindnodeattr(g, "shape");
                         agxset(n, a, "invtriangle");
                         a = agfindnodeattr(g, "label");
@@ -588,10 +606,8 @@ Agraph_t *gvplugin_graph(GVC_t * gvc)
                         break;
                     case API_layout:
 			/* draw textlayout  as hexagon - record last layout in plugin  (if any) in layout_n */
-                        strcpy(bufb, api_names[api]);
-                        strcat(bufb, "_");
-                        strcat(bufb, q);
-                        layout_n = n = agnode(ssg, bufb, 1);
+                        agxbprint(&buf, "%s_%s", api_names[api], q);
+                        layout_n = n = agnode(ssg, agxbuse(&buf), 1);
                         a = agfindnodeattr(g, "shape");
                         agxset(n, a, "hexagon");
                         a = agfindnodeattr(g, "label");
@@ -607,8 +623,8 @@ Agraph_t *gvplugin_graph(GVC_t * gvc)
             //    improve layout of cluster
             if (api == API_loadimage && !loadimage_n) {
 		neededge_loadimage = 1;
-                strcpy(buf2, "invis");
-                loadimage_n = n = agnode(ssg, bufa, 1);
+                agxbprint(&buf, "%s_%s_invis", package->name, api_names[api]);
+                loadimage_n = n = agnode(ssg, agxbuse(&buf), 1);
                 a = agfindnodeattr(g, "style");
                 agxset(n, a, "invis");
                 a = agfindnodeattr(g, "label");
@@ -616,8 +632,9 @@ Agraph_t *gvplugin_graph(GVC_t * gvc)
                 a = agfindnodeattr(g, "width");
                 agxset(n, a, "1.0");
 
-                strcpy(buf2, "invis_src");
-                n = agnode(g, bufa, 1);
+                agxbprint(&buf, "%s_%s_invis_src", package->name,
+                          api_names[api]);
+                n = agnode(g, agxbuse(&buf), 1);
                 a = agfindnodeattr(g, "style");
                 agxset(n, a, "invis");
                 a = agfindnodeattr(g, "label");
@@ -630,8 +647,8 @@ Agraph_t *gvplugin_graph(GVC_t * gvc)
             if (api == API_render && !renderer_n) {
 		neededge_loadimage = 1;
 		neededge_device = 1;
-                strcpy(buf2, "invis");
-                renderer_n = n = agnode(ssg, bufa, 1);
+                agxbprint(&buf, "%s_%s_invis", package->name, api_names[api]);
+                renderer_n = n = agnode(ssg, agxbuse(&buf), 1);
                 a = agfindnodeattr(g, "style");
                 agxset(n, a, "invis");
                 a = agfindnodeattr(g, "label");
@@ -639,8 +656,8 @@ Agraph_t *gvplugin_graph(GVC_t * gvc)
 	    }
             if (api == API_device && !device_n) {
 		neededge_device = 1;
-                strcpy(buf2, "invis");
-                device_n = n = agnode(ssg, bufa, 1);
+                agxbprint(&buf, "%s_%s_invis", package->name, api_names[api]);
+                device_n = n = agnode(ssg, agxbuse(&buf), 1);
                 a = agfindnodeattr(g, "style");
                 agxset(n, a, "invis");
                 a = agfindnodeattr(g, "label");
@@ -675,13 +692,7 @@ Agraph_t *gvplugin_graph(GVC_t * gvc)
     a = agfindgraphattr(ssg, "rank");
     agxset(ssg, a, "same");
     for (package = gvc->packages; package; package = package->next) {
-        strcpy(bufa, package->name);
-        strcat(bufa, "_");
-        buf1 = bufa + strlen(bufa);
         for (size_t api = 0; api < ARRAY_SIZE(api_names); api++) {
-            strcpy(buf1, api_names[api]);
-            strcat(buf1, "_");
-            buf2 = bufa + strlen(bufa);
             for (pnext = gvc->apis[api]; pnext; pnext = pnext->next) {
                 if (pnext->package == package) {
                     t = q = gv_strdup(pnext->typestr);
@@ -693,11 +704,11 @@ Agraph_t *gvplugin_graph(GVC_t * gvc)
 
  		    /* hack for aliases */
                     lq = q;
-                    if (!strncmp(q, "jp", 2)) {
+                    if (startswith(q, "jp")) {
                         q = "jpg";                /* canonical - for node name */
                         lq = "jpeg\\njpe\\njpg";  /* list - for label */
                     }
-                    else if (!strncmp(q, "tif", 3)) {
+                    else if (startswith(q, "tif")) {
                         q = "tif";
                         lq = "tiff\\ntif";
                     }
@@ -711,14 +722,15 @@ Agraph_t *gvplugin_graph(GVC_t * gvc)
                     }
 
                     switch (api) {
-                    case API_device:
-                        strcpy(buf2, q);
-                        n = agnode(g, bufa, 1);
-                        strcpy(bufb, "output_");
-                        strcat(bufb, q);
-                        m = agfindnode(ssg, bufb);
+                    case API_device: {
+                        agxbprint(&buf, "%s_%s_%s", package->name,
+                                  api_names[api], q);
+                        n = agnode(g, agxbuse(&buf), 1);
+                        agxbprint(&buf, "output_%s", q);
+                        char *const output = agxbuse(&buf);
+                        m = agfindnode(ssg, output);
                         if (!m) {
-                            m = agnode(ssg, bufb, 1);
+                            m = agnode(ssg, output, 1);
                             a = agfindnodeattr(g, "label");
                             agxset(m, a, lq);
                             a = agfindnodeattr(g, "shape");
@@ -728,24 +740,26 @@ Agraph_t *gvplugin_graph(GVC_t * gvc)
                         if (!e)
                             e = agedge(g, n, m, NULL, 1);
                         if (p && *p) {
-                            strcpy(bufb, "render_");
-                            strcat(bufb, p);
-                            m = agfindnode(ssg, bufb);
+                            agxbprint(&buf, "render_%s", p);
+                            char *const render = agxbuse(&buf);
+                            m = agfindnode(ssg, render);
                             if (!m)
-                                m = agnode(g, bufb, 1);
+                                m = agnode(g, render, 1);
                             e = agfindedge(g, m, n);
                             if (!e)
                                 e = agedge(g, m, n, NULL, 1);
                         }
                         break;
-                    case API_loadimage:
-                        strcpy(buf2, q);
-                        n = agnode(g, bufa, 1);
-                        strcpy(bufb, "input_");
-                        strcat(bufb, q);
-                        m = agfindnode(g, bufb);
+                    }
+                    case API_loadimage: {
+                        agxbprint(&buf, "%s_%s_%s", package->name,
+                                  api_names[api], q);
+                        n = agnode(g, agxbuse(&buf), 1);
+                        agxbprint(&buf, "input_%s", q);
+                        char *const input = agxbuse(&buf);
+                        m = agfindnode(g, input);
                         if (!m) {
-                            m = agnode(g, bufb, 1);
+                            m = agnode(g, input, 1);
                             a = agfindnodeattr(g, "label");
                             agxset(m, a, lq);
                             a = agfindnodeattr(g, "shape");
@@ -754,15 +768,16 @@ Agraph_t *gvplugin_graph(GVC_t * gvc)
                         e = agfindedge(g, m, n);
                         if (!e)
                             e = agedge(g, m, n, NULL, 1);
-                        strcpy(bufb, "render_");
-                        strcat(bufb, p);
-                        m = agfindnode(g, bufb);
+                        agxbprint(&buf, "render_%s", p);
+                        char *const render = agxbuse(&buf);
+                        m = agfindnode(g, render);
                         if (!m)
-                            m = agnode(g, bufb, 1);
+                            m = agnode(g, render, 1);
                         e = agfindedge(g, n, m);
                         if (!e)
                             e = agedge(g, n, m, NULL, 1);
                         break;
+                    }
                     default:
                         break;
                     }
@@ -772,5 +787,6 @@ Agraph_t *gvplugin_graph(GVC_t * gvc)
         }
     }
 
+    agxbfree(&buf);
     return g;
 }
