@@ -3,45 +3,51 @@
  * All rights reserved. This program and the accompanying materials
  * are made available under the terms of the Eclipse Public License v1.0
  * which accompanies this distribution, and is available at
- * http://www.eclipse.org/legal/epl-v10.html
+ * https://www.eclipse.org/legal/epl-v10.html
  *
  * Contributors: Details at https://graphviz.org
  *************************************************************************/
 
+#pragma once
+
+#include <stdlib.h>
+#include <util/list.h>
+
 #ifdef __cplusplus
 extern "C" {
 #endif
 
-#ifndef PARSE_H
-#define PARSE_H
-
     typedef enum { Begin =
 	    0, End, BeginG, EndG, Node, Edge, Eof, Error } case_t;
 
-    typedef struct _case_info {
+    typedef struct {
 	int gstart;
 	char *guard;
 	int astart;
 	char *action;
-	struct _case_info *next;
     } case_info;
 
-    typedef struct _parse_block {
+static inline void free_case_info(case_info c) {
+  free(c.guard);
+  free(c.action);
+}
+
+DEFINE_LIST_WITH_DTOR(case_infos, case_info, free_case_info)
+
+    typedef struct {
 	int l_beging;
 	char *begg_stmt;
-	int n_nstmts;
-	int n_estmts;
-	case_info *node_stmts;
-	case_info *edge_stmts;
-	struct _parse_block *next;
+	case_infos_t node_stmts;
+	case_infos_t edge_stmts;
     } parse_block; 
 
+DEFINE_LIST(parse_blocks, parse_block)
+
     typedef struct {
 	char *source;
 	int l_begin, l_end, l_endg;
 	char *begin_stmt;
-	int n_blocks;
-	parse_block *blocks;
+	parse_blocks_t blocks;
 	char *endg_stmt;
 	char *end_stmt;
     } parse_prog;
@@ -49,8 +55,6 @@ extern "C" {
     extern parse_prog *parseProg(char *, int);
     extern void freeParseProg (parse_prog *);
 
-#endif
-
 #ifdef __cplusplus
 }
 #endif
