@@ -3,7 +3,7 @@
  * All rights reserved. This program and the accompanying materials
  * are made available under the terms of the Eclipse Public License v1.0
  * which accompanies this distribution, and is available at
- * http://www.eclipse.org/legal/epl-v10.html
+ * https://www.eclipse.org/legal/epl-v10.html
  *
  * Contributors: Details at https://graphviz.org
  *************************************************************************/
@@ -15,8 +15,7 @@
 #include <neatogen/heap.h>
 #include <neatogen/voronoi.h>
 
-
-void voronoi(Site * (*nextsite) (void)) {
+void voronoi(Site *(*nextsite)(void *context), void *context) {
     Site *newsite, *bot, *top, *temp, *p;
     Site *v;
     Point newintstar = {0};
@@ -26,17 +25,17 @@ void voronoi(Site * (*nextsite) (void)) {
 
     edgeinit();
     siteinit();
-    PQinitialize();
-    bottomsite = nextsite();
+    pq_t *pq = PQinitialize();
+    bottomsite = nextsite(context);
     ELinitialize();
 
-    newsite = nextsite();
+    newsite = nextsite(context);
     while (1) {
-	if (!PQempty())
-	    newintstar = PQ_min();
+	if (!PQempty(pq))
+	    newintstar = PQ_min(pq);
 
 	if (newsite != NULL &&
-      (PQempty() || newsite->coord.y < newintstar.y ||
+      (PQempty(pq) || newsite->coord.y < newintstar.y ||
        (newsite->coord.y ==newintstar.y && newsite->coord.x < newintstar.x))) {
 	    /* new site is smallest */
 	    lbnd = ELleftbnd(&newsite->coord);
@@ -46,18 +45,18 @@ void voronoi(Site * (*nextsite) (void)) {
 	    bisector = HEcreate(e, le);
 	    ELinsert(lbnd, bisector);
 	    if ((p = hintersect(lbnd, bisector)) != NULL) {
-		PQdelete(lbnd);
-		PQinsert(lbnd, p, dist(p, newsite));
+		PQdelete(pq, lbnd);
+		PQinsert(pq, lbnd, p, ngdist(p, newsite));
 	    }
 	    lbnd = bisector;
 	    bisector = HEcreate(e, re);
 	    ELinsert(lbnd, bisector);
 	    if ((p = hintersect(bisector, rbnd)) != NULL)
-		PQinsert(bisector, p, dist(p, newsite));
-	    newsite = nextsite();
-	} else if (!PQempty()) {
+		PQinsert(pq, bisector, p, ngdist(p, newsite));
+	    newsite = nextsite(context);
+	} else if (!PQempty(pq)) {
 	    /* intersection is smallest */
-	    lbnd = PQextractmin();
+	    lbnd = PQextractmin(pq);
 	    llbnd = ELleft(lbnd);
 	    rbnd = ELright(lbnd);
 	    rrbnd = ELright(rbnd);
@@ -68,7 +67,7 @@ void voronoi(Site * (*nextsite) (void)) {
 	    endpoint(lbnd->ELedge, lbnd->ELpm, v);
 	    endpoint(rbnd->ELedge, rbnd->ELpm, v);
 	    ELdelete(lbnd);
-	    PQdelete(rbnd);
+	    PQdelete(pq, rbnd);
 	    ELdelete(rbnd);
 	    pm = le;
 	    if (bot->coord.y > top->coord.y) {
@@ -83,11 +82,11 @@ void voronoi(Site * (*nextsite) (void)) {
 	    endpoint(e, re - pm, v);
 	    deref(v);
 	    if ((p = hintersect(llbnd, bisector)) != NULL) {
-		PQdelete(llbnd);
-		PQinsert(llbnd, p, dist(p, bot));
+		PQdelete(pq, llbnd);
+		PQinsert(pq, llbnd, p, ngdist(p, bot));
 	    }
 	    if ((p = hintersect(bisector, rrbnd)) != NULL) {
-		PQinsert(bisector, p, dist(p, bot));
+		PQinsert(pq, bisector, p, ngdist(p, bot));
 	    }
 	} else
 	    break;
@@ -97,4 +96,8 @@ void voronoi(Site * (*nextsite) (void)) {
 	e = lbnd->ELedge;
 	clip_line(e);
     }
+
+    // `PQcleanup` relies on the number of sites, so should be discarded and
+    // at least every time we use `vAdjust`. See note in adjust.c:cleanup().
+    PQcleanup(pq);
 }
