@@ -3,7 +3,7 @@
  * All rights reserved. This program and the accompanying materials
  * are made available under the terms of the Eclipse Public License v1.0
  * which accompanies this distribution, and is available at
- * http://www.eclipse.org/legal/epl-v10.html
+ * https://www.eclipse.org/legal/epl-v10.html
  *
  * Contributors: Details at https://graphviz.org
  *************************************************************************/
@@ -13,7 +13,9 @@
 
 #define DEBUG
 
-#include <cgraph/alloc.h>
+#include <assert.h>
+#include <float.h>
+#include <limits.h>
 #include <math.h>
 #include <stdbool.h>
 #include <stddef.h>
@@ -21,8 +23,10 @@
 #include <ortho/partition.h>
 #include <ortho/trap.h>
 #include <common/arith.h>
+#include <util/alloc.h>
+#include <util/list.h>
 
-#define MARGIN 36;
+#define MARGIN 36
 
 #ifdef DEBUG
 char* pre = "%!PS-Adobe-2.0\n\
@@ -55,22 +59,21 @@ char* post = "showpage\n";
 
 /// @brief dumps @ref maze::gcells and @ref maze::cells via rects to PostScript
 
-static void
-psdump(cell *gcells, int n_gcells, boxf BB, boxf *rects, size_t nrect) {
+static void psdump(cell *gcells, size_t n_gcells, boxf BB, boxf *rects,
+                   size_t nrect) {
     boxf bb;
-    box absbb;
+    boxf absbb = {.LL = {.y = 10.0, .x = 10.0}};
 
-    absbb.LL.y = absbb.LL.x = 10;
     absbb.UR.x = absbb.LL.x + BB.UR.x - BB.LL.x;
     absbb.UR.y = absbb.LL.y + BB.UR.y - BB.LL.y;
     fputs (pre, stderr);
-    fprintf (stderr, "%%%%Page: 1 1\n%%%%PageBoundingBox: %d %d %d %d\n",
+    fprintf(stderr, "%%%%Page: 1 1\n%%%%PageBoundingBox: %.0f %.0f %.0f %.0f\n",
        absbb.LL.x, absbb.LL.y, absbb.UR.x, absbb.UR.y);
       
 
     fprintf (stderr, "%f %f translate\n", 10-BB.LL.x, 10-BB.LL.y);
     fputs ("0 0 1 setrgbcolor\n", stderr);
-    for (int i = 0; i < n_gcells; i++) {
+    for (size_t i = 0; i < n_gcells; i++) {
       bb = gcells[i].bb;
       fprintf (stderr, "%f %f %f %f node\n", bb.LL.x, bb.LL.y, bb.UR.x, bb.UR.y);
     }
@@ -87,11 +90,9 @@ psdump(cell *gcells, int n_gcells, boxf BB, boxf *rects, size_t nrect) {
 
 /// compares points by X and then by Y
 
-static int
-vcmpid(Dt_t* d, pointf* key1, pointf* key2, Dtdisc_t* disc)
-{
-  (void)d;
-  (void)disc;
+static int vcmpid(void *k1, void *k2) {
+  const pointf *key1 = k1;
+  const pointf *key2 = k2;
   int dx = dfp_cmp(key1->x, key2->x);
   if (dx != 0)
     return dx;
@@ -100,11 +101,9 @@ vcmpid(Dt_t* d, pointf* key1, pointf* key2, Dtdisc_t* disc)
 
 /// compares points by Y and then by X
 
-static int
-hcmpid(Dt_t* d, pointf* key1, pointf* key2, Dtdisc_t* disc)
-{
-  (void)d;
-  (void)disc;
+static int hcmpid(void *k1, void *k2) {
+  const pointf *key1 = k1;
+  const pointf *key2 = k2;
   int dy = dfp_cmp(key1->y, key2->y);
   if (dy != 0)
     return dy;
@@ -123,7 +122,7 @@ static Dtdisc_t vdictDisc = {
     offsetof(snodeitem,link),
     0,
     0,
-    (Dtcompar_f)vcmpid,
+    vcmpid,
 };
 static Dtdisc_t hdictDisc = {
     offsetof(snodeitem,p),
@@ -131,7 +130,7 @@ static Dtdisc_t hdictDisc = {
     offsetof(snodeitem,link),
     0,
     0,
-    (Dtcompar_f)hcmpid,
+    hcmpid,
 };
 
 #define delta 1        /* weight of length */
@@ -151,9 +150,7 @@ static Dtdisc_t hdictDisc = {
  * thinner the channel, the faster the weight rises.
  */
 
-static void
-updateWt (sedge* ep, int sz)
-{
+static void updateWt(sedge *ep, double sz) {
     ep->cnt++;
     if (ep->cnt > sz) {
 	ep->cnt = 0;
@@ -176,9 +173,9 @@ updateWts (sgraph* g, cell* cp, sedge* ep)
     int i;
     sedge* e;
     int isBend = BEND(g,ep);
-    int hsz = CHANSZ (cp->bb.UR.y - cp->bb.LL.y);
-    int vsz = CHANSZ (cp->bb.UR.x - cp->bb.LL.x);
-    int minsz = MIN(hsz, vsz);
+    const double hsz = CHANSZ(cp->bb.UR.y - cp->bb.LL.y);
+    const double vsz = CHANSZ(cp->bb.UR.x - cp->bb.LL.x);
+    const double minsz = fmin(hsz, vsz);
 
     /* Bend edges are added first */
     for (i = 0; i < cp->nedges; i++) {
@@ -324,6 +321,8 @@ chkSgraph (sgraph* g)
     
 }
 
+DEFINE_LIST(snodes, snode *)
+
 /**
  * @brief creates and fills @ref sgraph for @ref maze
  *
@@ -334,7 +333,7 @@ chkSgraph (sgraph* g)
 static sgraph*
 mkMazeGraph (maze* mp, boxf bb)
 {
-    int nsides, i, maxdeg;
+    int maxdeg;
     int bound = 4*mp->ncells;
     sgraph* g = createSGraph (bound + 2);
     Dt_t* vdict = dtopen(&vdictDisc,Dtoset);
@@ -347,7 +346,7 @@ mkMazeGraph (maze* mp, boxf bb)
      * a pointer to the cell.
      */
     sides = gv_calloc(4 * mp->ncells, sizeof(snode*));
-    for (i = 0; i < mp->ncells; i++) {
+    for (int i = 0; i < mp->ncells; i++) {
 	cell* cp = mp->cells+i;
         snode* np;
 	pointf pt;
@@ -384,47 +383,47 @@ mkMazeGraph (maze* mp, boxf bb)
      * connect it to its corresponding search nodes.
      */
     maxdeg = 0;
-    sides = gv_calloc(g->nnodes, sizeof(snode*));
-    nsides = 0;
-    for (i = 0; i < mp->ngcells; i++) {
-	cell* cp = mp->gcells+i;
+    for (size_t i = 0; i < mp->ngcells; i++) {
+	cell *cp = &mp->gcells[i];
         pointf pt; 
 	snodeitem* np;
 
-	cp->sides = sides+nsides;
+	snodes_t cp_sides = {0};
         pt = cp->bb.LL;
 	np = dtmatch (hdict, &pt);
 	for (; np && np->p.x < cp->bb.UR.x; np = dtnext (hdict, np)) {
-	    cp->sides[cp->nsides++] = np->np;
+	    snodes_append(&cp_sides, np->np);
 	    np->np->cells[1] = cp;
 	}
 	np = dtmatch (vdict, &pt);
 	for (; np && np->p.y < cp->bb.UR.y; np = dtnext (vdict, np)) {
-	    cp->sides[cp->nsides++] = np->np;
+	    snodes_append(&cp_sides, np->np);
 	    np->np->cells[1] = cp;
 	}
 	pt.y = cp->bb.UR.y;
 	np = dtmatch (hdict, &pt);
 	for (; np && np->p.x < cp->bb.UR.x; np = dtnext (hdict, np)) {
-	    cp->sides[cp->nsides++] = np->np;
+	    snodes_append(&cp_sides, np->np);
 	    np->np->cells[0] = cp;
 	}
 	pt.x = cp->bb.UR.x;
 	pt.y = cp->bb.LL.y;
 	np = dtmatch (vdict, &pt);
 	for (; np && np->p.y < cp->bb.UR.y; np = dtnext (vdict, np)) {
-	    cp->sides[cp->nsides++] = np->np;
+	    snodes_append(&cp_sides, np->np);
 	    np->np->cells[0] = cp;
 	}
-	nsides += cp->nsides;
+	assert(snodes_size(&cp_sides) <= INT_MAX);
+	cp->nsides = (int)snodes_size(&cp_sides);
+	cp->sides = snodes_detach(&cp_sides);
         if (cp->nsides > maxdeg) maxdeg = cp->nsides;
     }
 
     /* Mark cells that are small because of a small node, not because of the close
      * alignment of two rectangles.
      */
-    for (i = 0; i < mp->ngcells; i++) {
-	cell* cp = mp->gcells+i;
+    for (size_t i = 0; i < mp->ngcells; i++) {
+	cell *cp = &mp->gcells[i];
 	markSmall (cp);
     }
 
@@ -438,7 +437,7 @@ mkMazeGraph (maze* mp, boxf bb)
      * can have at most degree maxdeg.
      */
     initSEdges (g, maxdeg);
-    for (i = 0; i < mp->ncells; i++) {
+    for (int i = 0; i < mp->ncells; i++) {
 	cell* cp = mp->cells+i;
 	createSEdges (cp, g);
     }
@@ -462,13 +461,14 @@ maze *mkMaze(graph_t *g) {
     boxf* rects;
     cell* cp;
     double w2, h2;
-    boxf bb, BB;
+    boxf bb;
 
-    mp->ngcells = agnnodes(g);
+    assert(agnnodes(g) >= 0);
+    mp->ngcells = (size_t)agnnodes(g);
     cp = mp->gcells = gv_calloc(mp->ngcells, sizeof(cell));
 
-    BB.LL.x = BB.LL.y = MAXDOUBLE;
-    BB.UR.x = BB.UR.y = -MAXDOUBLE;
+    boxf BB = {.LL = {.x = DBL_MAX, .y = DBL_MAX},
+               .UR = {.x = -DBL_MAX, .y = -DBL_MAX}};
     for (n = agfstnode (g); n; n = agnxtnode(g,n)) {
         w2 = fmax(1, ND_xsize(n) / 2.0);
         h2 = fmax(1, ND_ysize(n) / 2.0);
@@ -491,7 +491,7 @@ maze *mkMaze(graph_t *g) {
     BB.UR.x += MARGIN;
     BB.UR.y += MARGIN;
     size_t nrect;
-    rects = partition (mp->gcells, mp->ngcells, &nrect, BB);
+    rects = partition(mp->gcells, mp->ngcells, &nrect, BB);
 
 #ifdef DEBUG
     if (odb_flags & ODB_MAZE) psdump (mp->gcells, mp->ngcells, BB, rects, nrect);
@@ -510,8 +510,10 @@ maze *mkMaze(graph_t *g) {
 void freeMaze (maze* mp)
 {
     free (mp->cells[0].sides);
-    free (mp->gcells[0].sides);
     free (mp->cells);
+    for (size_t i = 0; i < mp->ngcells; ++i) {
+	free(mp->gcells[i].sides);
+    }
     free (mp->gcells);
     freeSGraph (mp->sg);
     dtclose (mp->hchans);
