@@ -1,9 +1,9 @@
+#include <format>
 #include <memory>
 #include <stdexcept>
 
 #include <boost/algorithm/string.hpp>
-#include <catch2/catch.hpp>
-#include <fmt/format.h>
+#include <catch2/catch_all.hpp>
 
 #include "svg_analyzer.h"
 #include "svgpp_context.h"
@@ -129,7 +129,7 @@ SVG::SVGElement &SVGAnalyzer::current_element() {
   return *m_elements_in_process.back();
 }
 
-void SVGAnalyzer::enter_element(SVG::SVGElementType type) {
+void SVGAnalyzer::enter_element(SVG::SVGElementType elem_type) {
   if (m_elements_in_process.empty()) {
     throw std::runtime_error{
         "No element is currently being processed by the SVG++ document "
@@ -137,7 +137,7 @@ void SVGAnalyzer::enter_element(SVG::SVGElementType type) {
         "level 'svg' to be in process"};
   }
   auto &element = current_element();
-  element.children.emplace_back(type);
+  element.children.emplace_back(elem_type);
   m_elements_in_process.push_back(&element.children.back());
 }
 
@@ -151,7 +151,7 @@ void SVGAnalyzer::retrieve_graphviz_components() {
 
 void SVGAnalyzer::retrieve_graphviz_components_impl(
     SVG::SVGElement &svg_element) {
-  if (svg_element.type == SVG::SVGElementType::Group) {
+  if (svg_element.elem_type == SVG::SVGElementType::Group) {
     // The SVG 'class' attribute determines which type of Graphviz element a 'g'
     // element corresponds to
     const auto &class_ = svg_element.attributes.class_;
@@ -160,19 +160,19 @@ void SVGAnalyzer::retrieve_graphviz_components_impl(
     } else if (class_ == "node") {
       if (m_graphs.empty()) {
         throw std::runtime_error{
-            fmt::format("No graph to add node {} to", svg_element.graphviz_id)};
+            std::format("No graph to add node {} to", svg_element.graphviz_id)};
       }
       m_graphs.back().add_node(svg_element);
     } else if (class_ == "edge") {
       if (m_graphs.empty()) {
         throw std::runtime_error{
-            fmt::format("No graph to add edge {} to", svg_element.graphviz_id)};
+            std::format("No graph to add edge {} to", svg_element.graphviz_id)};
       }
       m_graphs.back().add_edge(svg_element);
     } else if (class_ == "cluster") {
       // ignore for now
     } else {
-      throw std::runtime_error{fmt::format("Unknown class {}", class_)};
+      throw std::runtime_error{std::format("Unknown class {}", class_)};
     }
   }
 
@@ -247,25 +247,25 @@ void SVGAnalyzer::set_rx(double rx) { current_element().attributes.rx = rx; }
 
 void SVGAnalyzer::set_ry(double ry) { current_element().attributes.ry = ry; }
 
-void SVGAnalyzer::set_point(std::pair<double, double> point) {
-  current_element().attributes.points.emplace_back(point.first, point.second);
+void SVGAnalyzer::set_point(std::pair<double, double> pt) {
+  current_element().attributes.points.emplace_back(pt.first, pt.second);
 }
 
 void SVGAnalyzer::set_text(std::string_view text) {
   auto &element = current_element();
   element.text = text;
 
-  if (element.type == SVG::SVGElementType::Title) {
+  if (element.elem_type == SVG::SVGElementType::Title) {
     // The title text is normally the 'graph_id', 'node_id' or 'edgeop'
     // according to the DOT language specification. Save it on the parent 'g'
     // element to avoid having to look it up later.
-    if (parent_element().type != SVG::SVGElementType::Group) {
+    if (parent_element().elem_type != SVG::SVGElementType::Group) {
       throw std::runtime_error{"Unexpected parent element of 'title' element"};
     }
     parent_element().graphviz_id = text;
     // If the 'g' element corresponds to the graph, set the Graphviz ID also on
     // the top level 'svg' element.
-    if (grandparent_element().type == SVG::SVGElementType::Svg) {
+    if (grandparent_element().elem_type == SVG::SVGElementType::Svg) {
       grandparent_element().graphviz_id = text;
     }
   }
@@ -283,6 +283,24 @@ void SVGAnalyzer::set_x(double x) { current_element().attributes.x = x; }
 
 void SVGAnalyzer::set_y(double y) { current_element().attributes.y = y; }
 
+void SVGAnalyzer::add_bboxes() {
+  for (auto &graph : m_graphs) {
+    graph.add_bboxes();
+  }
+}
+
+void SVGAnalyzer::add_node_edge_outline_bbox_overlaps(double tolerance) {
+  for (auto &graph : m_graphs) {
+    graph.add_node_edge_outline_bbox_overlaps(tolerance);
+  }
+}
+
+void SVGAnalyzer::add_outline_bboxes() {
+  for (auto &graph : m_graphs) {
+    graph.add_outline_bboxes();
+  }
+}
+
 SVGAnalyzer SVGAnalyzer::make_from_dot(const std::string &dot_source,
                                        const std::string &engine) {
   auto g = CGraph::AGraph{dot_source};
