@@ -8,15 +8,15 @@ from pathlib import Path
 import pytest
 
 sys.path.append(os.path.dirname(__file__))
-from gvtest import run_c  # pylint: disable=wrong-import-position
+from gvtest import compile_c, run, run_c  # pylint: disable=wrong-import-position
 
 
-@pytest.mark.parametrize("utility", ("bitarray", "list", "tokenize"))
+@pytest.mark.parametrize("utility", ("bitarray", "itos", "list", "tokenize"))
 def test_utility(utility: str):
     """run the given utility’s unit tests"""
 
     # locate the unit tests
-    src = Path(__file__).parent.resolve() / f"../lib/cgraph/test_{utility}.c"
+    src = Path(__file__).parent.resolve() / f"../lib/util/test_{utility}.c"
     assert src.exists()
 
     # locate lib directory that needs to be in the include path
@@ -25,7 +25,7 @@ def test_utility(utility: str):
     # extra C flags this compilation needs
     cflags = ["-I", lib]
     if platform.system() != "Windows":
-        cflags += ["-std=gnu99", "-Wall", "-Wextra", "-Werror"]
+        cflags += ["-std=gnu17"]
 
     _, _ = run_c(src, cflags=cflags)
 
@@ -46,6 +46,34 @@ def test_overflow_h(builtins: bool):
     if not builtins:
         cflags += ["-DSUPPRESS_BUILTINS"]
     if platform.system() != "Windows":
-        cflags += ["-std=gnu99", "-Wall", "-Wextra", "-Werror"]
+        cflags += ["-std=gnu17"]
 
     run_c(src, cflags=cflags)
+
+
+def test_gv_find_me(tmp_path: Path):
+    """test ../lib/util/gv_find_me.c::gv_find_me works correctly"""
+
+    # locate our support test file
+    src = Path(__file__).parent.resolve() / "test_gv_find_me.c"
+    assert src.exists()
+
+    # locate lib directory that needs to be in the include path
+    lib = Path(__file__).resolve().parents[1] / "lib"
+
+    # extra C flags this compilation needs
+    cflags = ["-I", lib]
+    if platform.system() != "Windows":
+        cflags += ["-std=gnu17"]
+
+    # pick a path to compile to
+    exe = tmp_path / "test_gv_find_me.exe"
+
+    _ = compile_c(src, cflags, dst=exe)
+
+    # run this
+    output = run([exe])
+
+    assert os.path.normpath(output) == os.path.normpath(
+        f"{exe}\n"
+    ), "gv_find_me did not determine executable absolute path"
